from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from re import match
from typing import Iterator

from camelot import read_pdf  # type: ignore


CATEGORY = r"^[MWF](PRO)?$"


class Title(Enum):
    CALENDAR: str = "PRO Event Calendar"
    START_LIST: str = "Start Lists"

    def __eq__(self, other) -> bool:
        return self.value.lower() == other.strip().lower()


@dataclass
class Event:
    when: str
    name: str
    prize: str
    slots: str
    registration: str
    deadline: str

    def __post_init__(self) -> None:
        self.date = datetime.strptime(self.when, "%m/%d/%Y")

    def __str__(self) -> str:
        fields = (
            self.date.strftime("%Y-%m-%d"),
            self.name,
            self.prize,
            self.slots,
            self.registration,
            self.deadline,
        )
        return "\t".join(fields)


@dataclass
class Athlete:
    bib: str
    first_name: str
    last_name: str
    country: str
    category: str

    @classmethod
    def from_row(cls, row: Iterator[str]) -> "Athlete":
        fields = {key: "" for key in cls.__annotations__}

        for field in (field.strip() for field in row):
            if field.isnumeric() and not fields["bib"]:
                fields["bib"] = field
            elif match(CATEGORY, field.upper()) and not fields["category"]:
                fields["category"] = field
            elif not fields["last_name"]:
                fields["last_name"] = field
            elif not fields["first_name"]:
                fields["first_name"] = field
            elif not fields["country"]:
                fields["country"] = field

        if not all(fields[key] for key in ("bib", "first_name", "last_name")):
            raise RuntimeError(f"Could not parse athlete from: {row}")

        return cls(**fields)

    @property
    def name(self) -> str:
        return f"{self.first_name} {self.last_name}"

    def __str__(self) -> str:
        fields = (self.bib, self.category, self.name, self.country)
        return "\t".join(field for field in fields if field)



import typer

from ironbot import scrappers
from ironbot.models import Title

app = typer.Typer()


@app.command()
def calendar() -> None:
    """List the details of the upcoming Ironman professional races."""
    data = scrappers.load(Title.CALENDAR)
    for event in scrappers.events(data):
        print(event)


@app.command()
def start_lists() -> None:
    """List upcoming Ironman professional races with start list available, with
    a unique number for each one (to be used in the `start-list` command)."""
    data = scrappers.load(Title.START_LIST)
    print("Choose one of the followign events to use with `start-list` command:")
    for number, name in enumerate(scrappers.event_names(data), 1):
        print(f" [{number}] {name}")
    return


@app.command()
def start_list(event_number: int) -> None:
    """Gets the start list for an Ironman professional race (use `start-lists`
    to get the event number)."""
    data = scrappers.load(Title.START_LIST)
    for athlete in scrappers.start_list(data, event_number):
        print(athlete)
